@@ -2,7 +2,7 @@
 #
 #   change-ws.rb -
 #   	$Release Version: 0.9.6$
-#   	$Revision: 56371 $
+#   	$Revision$
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -10,19 +10,26 @@
 #
 #
 
-require "irb/cmd/nop.rb"
-require "irb/ext/workspaces.rb"
+require_relative "nop"
+require_relative "../ext/workspaces"
 
-# :stopdoc:
 module IRB
+  # :stopdoc:
+
   module ExtendCommand
     class Workspaces < Nop
+      category "IRB"
+      description "Show workspaces."
+
       def execute(*obj)
         irb_context.workspaces.collect{|ws| ws.main}
       end
     end
 
     class PushWorkspace < Workspaces
+      category "IRB"
+      description "Push an object to the workspace stack."
+
       def execute(*obj)
         irb_context.push_workspace(*obj)
         super
@@ -30,12 +37,15 @@ module IRB
     end
 
     class PopWorkspace < Workspaces
+      category "IRB"
+      description "Pop a workspace from the workspace stack."
+
       def execute(*obj)
         irb_context.pop_workspace(*obj)
         super
       end
     end
   end
-end
-# :startdoc:
 
+  # :startdoc:
+end
