@@ -1,6 +1,7 @@
 # frozen_string_literal: true
-require_relative '../command'
-require_relative '../query_utils'
+
+require_relative "../command"
+require_relative "../query_utils"
 
 ##
 # Searches for gems starting with the supplied argument.
@@ -9,7 +10,7 @@ class Gem::Commands::ListCommand < Gem::Command
   include Gem::QueryUtils
 
   def initialize
-    super 'list', 'Display local gems whose name matches REGEXP',
+    super "list", "Display local gems whose name matches REGEXP",
          :domain => :local, :details => false, :versions => true,
          :installed => nil, :version => Gem::Requirement.default
 
