@@ -1,4 +1,3 @@
-# encoding: utf-8
 require "minitest/metametameta"
 require "stringio"
 
@@ -12,15 +11,12 @@ class ExampleA; end
 class ExampleB < ExampleA; end
 
 describe Minitest::Spec do
-  # helps to deal with 2.4 deprecation of Fixnum for Integer
-  Int = 1.class
-
   # do not parallelize this suite... it just can"t handle it.
 
   def assert_triggered expected = "blah", klass = Minitest::Assertion
     @assertion_count += 1
 
-    e = assert_raises(klass) do
+    e = assert_raises klass do
       yield
     end
 
@@ -29,17 +25,17 @@ describe Minitest::Spec do
     msg.gsub!(/(\d\.\d{6})\d+/, '\1xxx') # normalize: ruby version, impl, platform
     msg.gsub!(/:0x[Xa-fA-F0-9]{4,}[ @].+?>/, ":0xXXXXXX@PATH>")
 
-    if expected
+    return unless expected
+
+    @assertion_count += 1
+    case expected
+    when String then
+      assert_equal expected, msg
+    when Regexp then
       @assertion_count += 1
-      case expected
-      when String then
-        assert_equal expected, msg
-      when Regexp then
-        @assertion_count += 1
-        assert_match expected, msg
-      else
-        flunk "Unknown: #{expected.inspect}"
-      end
+      assert_match expected, msg
+    else
+      flunk "Unknown: #{expected.inspect}"
     end
   end
 
@@ -137,6 +133,46 @@ describe Minitest::Spec do
     end
   end
 
+  def good_pattern
+    capture_io do # 3.0 is noisy
+      eval "[1,2,3] => [Integer, Integer, Integer]" # eval to escape parser for ruby<3
+    end
+  end
+
+  def bad_pattern
+    capture_io do # 3.0 is noisy
+      eval "[1,2,3] => [Integer, Integer]" # eval to escape parser for ruby<3
+    end
+  end
+
+  it "needs to pattern match" do
+    @assertion_count = 1
+
+    if RUBY_VERSION > "3" then
+      expect { good_pattern }.must_pattern_match
+    else
+      assert_raises NotImplementedError do
+        expect {}.must_pattern_match
+      end
+    end
+  end
+
+  it "needs to error on bad pattern match" do
+    skip unless RUBY_VERSION > "3"
+
+    @assertion_count = 1
+
+    exp = if RUBY_VERSION.start_with? "3.0"
+            "[1, 2, 3]" # terrible error message!
+          else
+            /length mismatch/
+          end
+
+    assert_triggered exp do
+      expect { bad_pattern }.must_pattern_match
+    end
+  end
+
   it "needs to ensure silence" do
     @assertion_count -= 1 # no msg
     @assertion_count += 2 # assert_output is 2 assertions
@@ -156,7 +192,7 @@ describe Minitest::Spec do
     methods = Minitest::Expectations.public_instance_methods.grep(/must|wont/)
     methods.map!(&:to_s) if Symbol === methods.first
 
-    musts, wonts = methods.sort.partition { |m| m =~ /must/ }
+    musts, wonts = methods.sort.partition { |m| m.include? "must" }
 
     expected_musts = %w[must_be
                         must_be_close_to
@@ -172,6 +208,7 @@ describe Minitest::Spec do
                         must_include
                         must_match
                         must_output
+                        must_pattern_match
                         must_raise
                         must_respond_to
                         must_throw
@@ -179,7 +216,7 @@ describe Minitest::Spec do
 
     bad = %w[not raise throw send output be_silent]
 
-    expected_wonts = expected_musts.map { |m| m.sub(/must/, "wont") }.sort
+    expected_wonts = expected_musts.map { |m| m.sub("must", "wont") }.sort
     expected_wonts.reject! { |m| m =~ /wont_#{Regexp.union(*bad)}/ }
 
     _(musts).must_equal expected_musts
@@ -243,18 +280,14 @@ describe Minitest::Spec do
   end
 
   it "needs to warn on equality with nil" do
-    @assertion_count += 1 # extra test
+    @assertion_count = 3
+    @assertion_count += 2 unless error_on_warn? # 2 extra assertions
 
-    out, err = capture_io do
+    exp = /DEPRECATED: Use assert_nil if expecting nil from .* This will fail in Minitest 6./
+
+    assert_deprecation exp do
       assert_success _(nil).must_equal(nil)
     end
-
-    exp = "DEPRECATED: Use assert_nil if expecting nil from #{__FILE__}:#{__LINE__-3}. " \
-      "This will fail in Minitest 6.\n"
-    exp = "" if $-w.nil?
-
-    assert_empty out
-    assert_equal exp, err
   end
 
   it "needs to verify floats outside a delta" do
@@ -362,12 +395,12 @@ describe Minitest::Spec do
   it "needs to verify instances of a class" do
     assert_success _(42).wont_be_instance_of(String)
 
-    assert_triggered "Expected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int
+    assert_triggered "Expected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer
     end
 
-    assert_triggered "msg.\nExpected 42 to not be an instance of #{Int.name}." do
-      _(42).wont_be_instance_of Int, "msg"
+    assert_triggered "msg.\nExpected 42 to not be an instance of Integer." do
+      _(42).wont_be_instance_of Integer, "msg"
     end
   end
 
@@ -377,26 +410,26 @@ describe Minitest::Spec do
     assert_success _(42).wont_be_kind_of(String)
     assert_success _(proc {}).wont_be_kind_of(String)
 
-    assert_triggered "Expected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int
+    assert_triggered "Expected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer
     end
 
-    assert_triggered "msg.\nExpected 42 to not be a kind of #{Int.name}." do
-      _(42).wont_be_kind_of Int, "msg"
+    assert_triggered "msg.\nExpected 42 to not be a kind of Integer." do
+      _(42).wont_be_kind_of Integer, "msg"
     end
   end
 
   it "needs to verify kinds of objects" do
     @assertion_count += 3 # extra test
 
-    assert_success _(6 * 7).must_be_kind_of(Int)
+    assert_success _(6 * 7).must_be_kind_of(Integer)
     assert_success _(6 * 7).must_be_kind_of(Numeric)
 
-    assert_triggered "Expected 42 to be a kind of String, not #{Int.name}." do
+    assert_triggered "Expected 42 to be a kind of String, not Integer." do
       _(6 * 7).must_be_kind_of String
     end
 
-    assert_triggered "msg.\nExpected 42 to be a kind of String, not #{Int.name}." do
+    assert_triggered "msg.\nExpected 42 to be a kind of String, not Integer." do
       _(6 * 7).must_be_kind_of String, "msg"
     end
 
@@ -535,7 +568,8 @@ describe Minitest::Spec do
 
     it "can NOT use must_equal in a thread. It must use expect in a thread" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
-      assert_raises RuntimeError do
+
+      assert_raises RuntimeError, Minitest::UnexpectedWarning do
         capture_io do
           Thread.new { (1 + 1).must_equal 2 }.join
         end
@@ -545,29 +579,33 @@ describe Minitest::Spec do
     it "fails gracefully when expectation used outside of `it`" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 1
+      @assertion_count += 2 # assert_match is compound
 
-      e = assert_raises RuntimeError do
+      e = assert_raises RuntimeError, Minitest::UnexpectedWarning do
         capture_io do
           Thread.new { # forces ctx to be nil
-            describe("woot") do
+            describe "woot" do
               (1 + 1).must_equal 2
             end
           }.join
         end
       end
 
-      assert_equal "Calling #must_equal outside of test.", e.message
+      exp = "Calling #must_equal outside of test."
+      exp = "DEPRECATED: global use of must_equal from" if error_on_warn?
+
+      assert_match exp, e.message
     end
 
     it "deprecates expectation used without _" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 3
+      @assertion_count += 1
+      @assertion_count += 2 unless error_on_warn?
 
       exp = /DEPRECATED: global use of must_equal from/
 
-      assert_output "", exp do
+      assert_deprecation exp do
         (1 + 1).must_equal 2
       end
     end
@@ -577,12 +615,13 @@ describe Minitest::Spec do
     it "deprecates expectation used without _ with empty backtrace_filter" do
       skip "N/A" if ENV["MT_NO_EXPECTATIONS"]
 
-      @assertion_count += 3
+      @assertion_count += 1
+      @assertion_count += 2 unless error_on_warn?
 
       exp = /DEPRECATED: global use of must_equal from/
 
       with_empty_backtrace_filter do
-        assert_output "", exp do
+        assert_deprecation exp do
           (1 + 1).must_equal 2
         end
       end
@@ -613,9 +652,9 @@ describe Minitest::Spec do
   end
 
   it "needs to verify types of objects" do
-    assert_success _(6 * 7).must_be_instance_of(Int)
+    assert_success _(6 * 7).must_be_instance_of(Integer)
 
-    exp = "Expected 42 to be an instance of String, not #{Int.name}."
+    exp = "Expected 42 to be an instance of String, not Integer."
 
     assert_triggered exp do
       _(6 * 7).must_be_instance_of String
@@ -642,7 +681,7 @@ describe Minitest::Spec do
     assert_success _(41).must_be(:<, 42)
 
     assert_triggered "Expected 42 to be < 41." do
-      _(42).must_be(:<, 41)
+      _(42).must_be :<, 41
     end
   end
 
@@ -659,11 +698,11 @@ describe Minitest::Spec do
   it "needs to verify using respond_to" do
     assert_success _(42).must_respond_to(:+)
 
-    assert_triggered "Expected 42 (#{Int.name}) to respond to #clear." do
+    assert_triggered "Expected 42 (Integer) to respond to #clear." do
       _(42).must_respond_to :clear
     end
 
-    assert_triggered "msg.\nExpected 42 (#{Int.name}) to respond to #clear." do
+    assert_triggered "msg.\nExpected 42 (Integer) to respond to #clear." do
       _(42).must_respond_to :clear, "msg"
     end
   end
@@ -710,9 +749,9 @@ describe Minitest::Spec, :let do
   it "doesn't raise an error if it is just another let" do
     v = proc do
       describe :outer do
-        let(:bar)
+        let :bar
         describe :inner do
-          let(:bar)
+          let :bar
         end
       end
       :good
@@ -975,8 +1014,9 @@ class TestMeta < MetaMetaMetaTestCase
       z = describe "second thingy" do end
     end
 
-    test_methods = ["test_0001_top level it",
-                    "test_0002_не латинские &いった α, β, γ, δ, ε hello!!! world",
+    test_methods = [
+                     "test_0001_top level it",
+                     "test_0002_не латинские &いった α, β, γ, δ, ε hello!!! world",
                    ].sort
 
     assert_equal test_methods, [x1, x2]
@@ -1068,3 +1108,38 @@ class ValueMonadTest < Minitest::Test
     assert_equal "c", struct.expect
   end
 end
+
+describe Minitest::Spec, :infect_an_assertion do
+  class << self
+    attr_accessor :infect_mock
+  end
+
+  def assert_infects exp, act, msg = nil, foo: nil, bar: nil
+    self.class.infect_mock.assert_infects exp, act, msg, foo: foo, bar: bar
+  end
+
+  infect_an_assertion :assert_infects, :must_infect
+  infect_an_assertion :assert_infects, :must_infect_without_flipping, :dont_flip
+
+  it "infects assertions with kwargs" do
+    mock = Minitest::Mock.new
+    mock.expect :assert_infects, true, [:exp, :act, nil], foo: :foo, bar: :bar
+
+    self.class.infect_mock = mock
+
+    _(:act).must_infect :exp, foo: :foo, bar: :bar
+
+    assert_mock mock
+  end
+
+  it "infects assertions with kwargs (dont_flip)" do
+    mock = Minitest::Mock.new
+    mock.expect :assert_infects, true, [:act, :exp, nil], foo: :foo, bar: :bar
+
+    self.class.infect_mock = mock
+
+    _(:act).must_infect_without_flipping :exp, foo: :foo, bar: :bar
+
+    assert_mock mock
+  end
+end
