@@ -3,7 +3,7 @@
 # Author:: Nathaniel Talbott.
 # Copyright::
 #   * Copyright (c) 2000-2003 Nathaniel Talbott. All rights reserved.
-#   * Copyright (c) 2008-2013 Kouhei Sutou <kou@clear-code.com>
+#   * Copyright (c) 2008-2017 Kouhei Sutou <kou@clear-code.com>
 # License:: Ruby license.
 
 begin
@@ -110,8 +110,10 @@ module Test
           end
 
           def finished(elapsed_time)
-            nl if output?(NORMAL) and !output?(VERBOSE)
-            output_faults unless @show_detail_immediately
+            unless @show_detail_immediately
+              nl if output?(NORMAL) and !output?(VERBOSE)
+              output_faults
+            end
             nl(PROGRESS_ONLY)
             change_output_level(IMPORTANT_FAULTS_ONLY) do
               output_statistics(elapsed_time)
@@ -178,21 +180,16 @@ module Test
 
           def output_fault_in_detail(fault)
             if fault.is_a?(Failure) and
-                fault.inspected_expected and fault.inspected_actual
+                fault.inspected_expected and
+                fault.inspected_actual
               output_single("#{fault.label}: ")
               output(fault.test_name, fault_color(fault))
               output_fault_backtrace(fault)
               output_failure_message(fault)
             else
-              if fault.is_a?(Error)
-                output_single("#{fault.label}: ")
-                output_single(fault.test_name, fault_color(fault))
-                output_fault_message(fault)
-              else
-                output_single(fault.label, fault_color(fault))
-                output_fault_message(fault)
-                output(fault.test_name)
-              end
+              output_single("#{fault.label}: ")
+              output_single(fault.test_name, fault_color(fault))
+              output_fault_message(fault)
               output_fault_backtrace(fault)
             end
           end
@@ -212,26 +209,38 @@ module Test
           end
 
           def output_fault_backtrace(fault)
-            snippet_is_shown = false
             detector = FaultLocationDetector.new(fault, @code_snippet_fetcher)
             backtrace = fault.location
             # workaround for test-spec. :<
             # see also GitHub:#22
             backtrace ||= []
+
+            code_snippet_backtrace_index = nil
+            code_snippet_lines = nil
             backtrace.each_with_index do |entry, i|
-              output(entry)
-              next if snippet_is_shown
               next unless detector.target?(entry)
               file, line_number, = detector.split_backtrace_entry(entry)
-              snippet_is_shown = output_code_snippet(file, line_number,
-                                                     fault_color(fault))
+              lines = fetch_code_snippet(file, line_number)
+              unless lines.empty?
+                code_snippet_backtrace_index = i
+                code_snippet_lines = lines
+                break
+              end
+            end
+
+            backtrace.each_with_index do |entry, i|
+              output(entry)
+              if i == code_snippet_backtrace_index
+                output_code_snippet(code_snippet_lines, fault_color(fault))
+              end
             end
           end
 
-          def output_code_snippet(file, line_number, target_line_color=nil)
-            lines = @code_snippet_fetcher.fetch(file, line_number)
-            return false if lines.empty?
+          def fetch_code_snippet(file, line_number)
+            @code_snippet_fetcher.fetch(file, line_number)
+          end
 
+          def output_code_snippet(lines, target_line_color=nil)
             max_n = lines.collect {|n, line, attributes| n}.max
             digits = (Math.log10(max_n) + 1).truncate
             lines.each do |n, line, attributes|
@@ -245,7 +254,6 @@ module Test
               output("  %2s %*d: %s" % [current_line_mark, digits, n, line],
                      line_color)
             end
-            true
           end
 
           def output_failure_message(failure)
@@ -303,6 +311,7 @@ module Test
           end
 
           def output_fault_in_short(fault)
+            output_single("#{fault.label}: ")
             output_single(fault.message, fault_color(fault))
             output(" [#{fault.test_name}]")
             output(fault.location.first)
@@ -330,24 +339,34 @@ module Test
           end
 
           def output_summary_marker
-            term_width = guess_term_width
-            if term_width.zero?
-              marker_width = 6
+            if @progress_row_max > 0
+              output("-" * @progress_row_max, summary_marker_color)
             else
-              marker_width = term_width
+              nl
             end
-            output("-" * marker_width, summary_marker_color)
           end
 
           def test_started(test)
             return unless output?(VERBOSE)
 
-            name = test.name.sub(/\(.+?\)\z/, '')
-            right_space = 8 * 2
+            tab_width = 8
+            name = test.local_name
+            separator = ":"
+            left_used = indent.size + name.size + separator.size
+            right_space = tab_width * 2
             left_space = @progress_row_max - right_space
-            left_space = left_space - indent.size - name.size
-            tab_stop = "\t" * ([left_space - 1, 0].max / 8)
-            output_single("#{indent}#{name}:#{tab_stop}", nil, VERBOSE)
+            if (left_used % tab_width).zero?
+              left_space -= left_used
+              n_tabs = 0
+            else
+              left_space -= ((left_used / tab_width) + 1) * tab_width
+              n_tabs = 1
+            end
+            n_tabs += [left_space, 0].max / tab_width
+            tab_stop = "\t" * n_tabs
+            output_single("#{indent}#{name}#{separator}#{tab_stop}",
+                          nil,
+                          VERBOSE)
             @test_start = Time.now
           end
 
@@ -416,12 +435,14 @@ module Test
 
           def output_single(something, color=nil, level=nil)
             return false unless output?(level)
-            if @use_color and color
-              something = "%s%s%s" % [color.escape_sequence,
-                                      something,
-                                      @reset_color.escape_sequence]
+            something.to_s.each_line do |line|
+              if @use_color and color
+                line = "%s%s%s" % [color.escape_sequence,
+                                   line,
+                                   @reset_color.escape_sequence]
+              end
+              @output.write(line)
             end
-            @output.write(something)
             @output.flush
             true
           end
@@ -473,8 +494,12 @@ module Test
             fault_class.name.split(/::/).last.downcase
           end
 
+          def fault_class_color(fault_class)
+            color(fault_class_color_name(fault_class))
+          end
+
           def fault_color(fault)
-            color(fault_class_color_name(fault.class))
+            fault_class_color(fault.class)
           end
 
           def fault_marker_color(fault)
@@ -485,12 +510,30 @@ module Test
             color("#{@result.status}-marker")
           end
 
+          TERM_COLOR_SUPPORT = /
+            color|  # explicitly claims color support in the name
+            direct| # explicitly claims "direct color" (24 bit) support
+            #{ColorScheme::TERM_256}|
+            \Acygwin|
+            \Alinux|
+            \Ansterm-bce|
+            \Ansterm-c-|
+            \Aputty|
+            \Arxvt|
+            \Ascreen|
+            \Atmux|
+            \Axterm
+          /x
+
           def guess_color_availability
+            return true if ENV["GITHUB_ACTIONS"] == "true"
             return false unless @output.tty?
             return true if windows? and ruby_2_0_or_later?
             case ENV["TERM"]
             when /(?:term|screen)(?:-(?:256)?color)?\z/
               true
+            when TERM_COLOR_SUPPORT
+              true
             else
               return true if ENV["EMACS"] == "t"
               false
@@ -620,6 +663,7 @@ module Test
 
           def diff_line(from_line, to_line)
             to_operations = []
+            mark_operations = []
             from_line, to_line, _operations = line_operations(from_line, to_line)
 
             no_replace = true
@@ -649,11 +693,22 @@ module Test
                     output_single(" " * (from_width - to_width))
                   end
                 end
+                mark_operations << Proc.new do
+                  output_single("?" * from_width,
+                                color("diff-difference-tag"))
+                  if (to_width < from_width)
+                    output_single(" " * (from_width - to_width))
+                  end
+                end
               when :delete
                 output_single(from_line[from_start...from_end],
                               color("diff-deleted"))
                 unless no_replace
                   to_operations << Proc.new {output_single(" " * from_width)}
+                  mark_operations << Proc.new do
+                    output_single("-" * from_width,
+                                  color("diff-deleted"))
+                  end
                 end
               when :insert
                 if no_replace
@@ -665,11 +720,16 @@ module Test
                     output_single(to_line[to_start...to_end],
                                   color("diff-inserted"))
                   end
+                  mark_operations << Proc.new do
+                    output_single("+" * to_width,
+                                  color("diff-inserted"))
+                  end
                 end
               when :equal
                 output_single(from_line[from_start...from_end])
                 unless no_replace
                   to_operations << Proc.new {output_single(" " * to_width)}
+                  mark_operations << Proc.new {output_single(" " * to_width)}
                 end
               else
                 raise "unknown tag: #{tag}"
@@ -685,6 +745,15 @@ module Test
               end
               output("")
             end
+
+            unless mark_operations.empty?
+              output_single("?", color("diff-difference-tag"))
+              output_single(" ")
+              mark_operations.each do |operation|
+                operation.call
+              end
+              output("")
+            end
           end
         end
       end
